@@ -251,6 +251,7 @@ sub profile_default
 	my $def = $_[ 0]-> SUPER::profile_default;
 	my %prf = (
 		file  => undef,
+		fd    => -1,
 		mask  => fe::Read | fe::Write | fe::Exception,
 		owner => undef,
 	);
@@ -690,6 +691,7 @@ my %RNT = (
 	Show           => nt::Default,
 	Size           => nt::Default,
 	TranslateAccel => nt::Default,
+	SysHandle      => nt::Default,
 	ZOrderChanged  => nt::Default,
 );
 
@@ -1199,6 +1201,7 @@ sub profile_default
 		borderStyle           => bs::Sizeable,
 		clipOwner             => 0,
 		growMode              => gm::DontCare,
+		effects               => undef,
 		icon                  => 0,
 		mainWindow            => 0,
 		menu                  => undef,
@@ -1494,6 +1497,7 @@ my %RNT = (
 	PasteText   => nt::Action,
 	CopyImage   => nt::Action,
 	PasteImage  => nt::Action,
+	Idle        => nt::Default,
 );
 
 sub notification_types { return \%RNT; }
