@@ -308,6 +308,7 @@ sub item_sort {
 	for my $i (@$items) {
 		if (defined $i->[1]) { $self->item_sort($i->[1], $col, $dir) }
 	}
+	$self->reset_item_cache;
 }
 
 sub on_expand {
@@ -321,8 +322,6 @@ sub autoRecalc {($#_)?$_[0]->set_auto_recalc ($_[1]):return $_[0]->{autoRecalc}
 
 1;
 
-__DATA__
-
 =pod
 
 =head1 NAME
