@@ -86,7 +86,7 @@ prima_prepare_ximage( int width, int height, int format)
 		pformat = XYBitmap;
 		break;
 	case CACHE_LAYERED:
-		if ( guts. argb_pic_format ) {
+		if ( guts. argb_depth ) {
 			depth   = guts. argb_visual.depth;
 			idepth  = guts. argb_depth;
 			visual  = guts. argb_visual. visual;
@@ -135,7 +135,7 @@ prima_prepare_ximage( int width, int height, int format)
 		if ( i-> xmem. shmaddr == (void*)-1 || i-> xmem. shmaddr == nil) {
 			i-> image-> data = nil;
 			XDestroyImage( i-> image);
-			shmctl( i-> xmem. shmid, IPC_RMID, 0);
+			shmctl( i-> xmem. shmid, IPC_RMID, NULL);
 			goto normal_way;
 		}
 		i-> xmem. readOnly = false;
@@ -148,14 +148,14 @@ bad_xshm_attach:
 			i-> image-> data = nil;
 			XDestroyImage( i-> image);
 			shmdt( i-> xmem. shmaddr);
-			shmctl( i-> xmem. shmid, IPC_RMID, 0);
+			shmctl( i-> xmem. shmid, IPC_RMID, NULL);
 			goto normal_way;
 		}
 		XCHECKPOINT;
 		XSync(DISP,false);
 		XCHECKPOINT;
 		if (guts.xshmattach_failed)       goto bad_xshm_attach;
-		shmctl( i-> xmem. shmid, IPC_RMID, 0);
+		shmctl( i-> xmem. shmid, IPC_RMID, NULL);
 		i-> data_alias = i-> image-> data;
 		i-> shm = true;
 		return i;
@@ -413,7 +413,7 @@ apc_dbm_create( Handle self, int type)
 		depth = 1;
 		break;
 	case dbtLayered:
-		if ( guts. argb_pic_format ) {
+		if ( guts. argb_depth ) {
 			XX-> flags.layered = 1;
 			depth = guts. argb_depth;
 			XX-> colormap = guts. argbColormap;
@@ -438,7 +438,7 @@ apc_dbm_create( Handle self, int type)
 
 #ifdef HAVE_X11_EXTENSIONS_XRENDER_H
 	if ( XF_LAYERED(XX) )
-		XX->argb_picture = XRenderCreatePicture( DISP, XX->gdrawable, guts. argb_pic_format, 0, NULL);
+		XX->argb_picture = XRenderCreatePicture( DISP, XX->gdrawable, guts. xrender_argb_pic_format, 0, NULL);
 #endif
 
 	return true;
@@ -507,13 +507,12 @@ prima_copy_xybitmap( unsigned char *data, const unsigned char *idata, int w, int
 		}
 	} else {
 		mirrored_bits = mirror_bits();
-		w = ( w + 7) / 8;
 		for ( y = h-1; y >= 0; y--) {
 			register const unsigned char *s = idata+y*ils;
 			register unsigned char *t = ls*(h-y-1)+data;
-			for ( x = 0; x < w; x++) {
+			x = ls;
+			while (x--)
 				*t++ = mirrored_bits[*s++];
-			}
 		}
 	}
 }
@@ -1717,7 +1716,7 @@ img_put_layered_on_bitmap( Handle self, Handle image, PutImageRequest * req)
 	ok = prima_query_argb_rect( obj, X(image)-> gdrawable, req-> src_x, req-> src_y, req-> w, req-> h);
 	if ( !ok ) {
 		Object_destroy( obj );
-		return nilHandle;
+		return false;
 	}
 
 	req->src_x = req->src_y = 0;
@@ -1802,7 +1801,7 @@ img_put_layered_on_pixmap( Handle self, Handle image, PutImageRequest * req)
 	PDrawableSysData YY = X(image);
 	Picture target;
 
-	target  = XRenderCreatePicture( DISP, XX->gdrawable, guts. argb_compat_format, 0, NULL);
+	target  = XRenderCreatePicture( DISP, XX->gdrawable, guts. xrender_argb_compat_format, 0, NULL);
 	XRenderComposite( DISP, (req-> rop == ropSrcCopy) ? PictOpSrc : PictOpOver, YY-> argb_picture, 0, target,
 		req->src_x, req->src_y, 0, 0,
 		req->dst_x, req->dst_y, req->w, req->h
@@ -1895,7 +1894,7 @@ img_put_pixmap_on_layered( Handle self, Handle image, PutImageRequest * req)
 
 	if ( render_rop >= PictOpMinimum ) {
 		/* cheap on-server blit */
-		picture = XRenderCreatePicture( DISP, YY->gdrawable, guts. argb_compat_format, 0, NULL);
+		picture = XRenderCreatePicture( DISP, YY->gdrawable, guts. xrender_argb_compat_format, 0, NULL);
 		XRenderComposite( DISP, render_rop, picture, 0, XX-> argb_picture,
 			req->src_x, req->src_y, 0, 0,
 			req->dst_x, req->dst_y, req->w, req->h
@@ -1949,8 +1948,8 @@ img_put_argb_on_pixmap_or_widget( Handle self, Handle image, PutImageRequest * r
 		req->w, req->h
 	))) goto FAIL;
 
-	picture = XRenderCreatePicture( DISP, pixmap, guts. argb_pic_format, 0, NULL);
-	target  = XRenderCreatePicture( DISP, XX->gdrawable, guts. argb_compat_format, 0, NULL);
+	picture = XRenderCreatePicture( DISP, pixmap, guts. xrender_argb_pic_format, 0, NULL);
+	target  = XRenderCreatePicture( DISP, XX->gdrawable, guts. xrender_argb_compat_format, 0, NULL);
 	XRenderComposite( DISP, (req-> rop == ropSrcCopy) ? PictOpSrc : PictOpOver, picture, 0, target,
 		0, 0, 0, 0,
 		req->dst_x, req->dst_y, req->w, req->h
@@ -2042,7 +2041,7 @@ img_put_argb_on_layered( Handle self, Handle image, PutImageRequest * req)
 		req->w, req->h
 	))) goto FAIL;
 
-	picture = XRenderCreatePicture( DISP, pixmap, guts. argb_pic_format, 0, NULL);
+	picture = XRenderCreatePicture( DISP, pixmap, guts. xrender_argb_pic_format, 0, NULL);
 	XRenderComposite( DISP, (req-> rop == ropSrcCopy) ? PictOpSrc : PictOpOver, picture, 0, XX-> argb_picture,
 		0, 0, 0, 0,
 		req->dst_x, req->dst_y, req->w, req->h
@@ -2224,7 +2223,7 @@ apc_image_begin_paint( Handle self)
 	XX-> colormap    = guts. defaultColormap;
 #ifdef HAVE_X11_EXTENSIONS_XRENDER_H
 	if ( XF_LAYERED(XX) ) {
-		XX->argb_picture = XRenderCreatePicture( DISP, XX->gdrawable, guts. argb_pic_format, 0, NULL);
+		XX-> argb_picture = XRenderCreatePicture( DISP, XX->gdrawable, guts. xrender_argb_pic_format, 0, NULL);
 		XX-> visual      = &guts. argb_visual;
 		XX-> colormap    = guts. argbColormap;
 	}
@@ -2603,7 +2602,7 @@ prima_query_argb_rect( Handle self, Pixmap px, int x, int y, int w, int h)
 		break;
 slurp_image_unsupported_depth:
 	default:
-		warn("UAI_023: unsupported backing image conversion from %d to %d\n", guts.argb_depth, guts. qdepth);
+		warn("UAI_023: unsupported backing image conversion from %d to %d\n", guts. argb_depth, guts. qdepth);
 		return false;
 	}
 	
@@ -2686,8 +2685,8 @@ apc_image_end_paint( Handle self)
 
 Bool
 apc_gp_stretch_image( Handle self, Handle image,
-							int dst_x, int dst_y, int src_x, int src_y,
-							int dst_w, int dst_h, int src_w, int src_h, int rop)
+	int dst_x, int dst_y, int src_x, int src_y,
+	int dst_w, int dst_h, int src_w, int src_h, int rop)
 {
 	DEFXX;
 	PDrawableSysData YY = X(image);
@@ -2747,7 +2746,9 @@ apc_gp_stretch_image( Handle self, Handle image,
 
 		if ( XT_IS_ICON(YY)) {
 			int height = src_w, width = src_h;
-			PIcon isrc = (PIcon) image, idst = (PIcon) obj;
+			PIcon isrc = (PIcon) image, idst;
+			obj = ( Handle) create_object("Prima::Icon", "");
+			idst = (PIcon) obj;
 			CIcon( obj)-> create_empty_icon( obj, src_w, src_h,
 				(src == SRC_BITMAP) ? imBW : guts. qdepth,
 				isrc-> maskType
